import os
from PyQt5.QtCore import QSettings
from peewee import SqliteDatabase, Model

from qtradingview.utils import AppUtil


# genera el objeto db y lo devuelve
def get_db():
    settings = QSettings("QTradingView", "Settings")
    dirname = os.path.dirname(settings.fileName())
    nou_path = os.path.join(dirname, 'database.db')
    db_file_path = AppUtil.get_db_file_path()
    db = SqliteDatabase(nou_path, pragmas={
        'journal_mode': 'wal',
        'cache_size': -1 * 64000,  # 64MB
        'foreign_keys': 1,
        'ignore_check_constraints': 0,
        'synchronous': 0})
    return db


def migrate(db):
    from peewee_migrate import Router
    router = Router(db)
    # Create migration
    router.create('migration_name', auto=True)
    # Run migration/migrations
    router.run('migration_name')
    # Run all unapplied migrations
    router.run()


# ─── CUSTOM MODEL BASE ──────────────────────────────────────────────────────────

class CustomModel(Model):

    class Meta:
        database = get_db()

    @classmethod
    def count_all(cls):
        return cls.select().count()

# ────────────────────────────────────────────────────────────────────────────────



from PyQt5 import QtWidgets, QtCore
from PyQt5.QtCore import Qt, QLocale

from qtradingview.ui.dock_portfolio_Ui import Ui_DockPortfolio
from .dialog import DialogTrade
from .models import Trades
from .widgets import TradesTableModel, CustomContextMenu


# ─── DOCK PORTFOLIO ─────────────────────────────────────────────────────────────

class DockPortfolio(QtWidgets.QDockWidget, Ui_DockPortfolio):

    def __init__(self, parent=None):
        QtWidgets.QDockWidget.__init__(self, parent=parent)
        self.setupUi(self)
        self.mw = parent
        self.setWindowTitle("Portfolio")
        #
        self.refresh_timer = QtCore.QTimer(self)
        self.refresh_timer.setInterval(10000)
        self._signals()
        self.setVisible(self.mw.actionPortfolio.isChecked())

    def onActionEvent(self, actived):
        """ Show/hide this dock and raise if actived """
        self.setVisible(actived)
        if actived:
            self.raise_()

    def closeEvent(self, event):
        self.mw.actionPortfolio.setChecked(False)
        self.setVisible(False)

    def setVisible(self, active):
        """Show/hide this dock and start/stop refreshing table timer"""
        self.refresh_timer.start() if active else self.refresh_timer.stop()
        return super().setVisible(active)

    def _signals(self):
        self.tabla.customContextMenuRequested.connect(self.contextoMenuEvent)
        self.refresh_timer.timeout.connect(self.refreshTable)
        self.combo_trades.currentTextChanged.connect(self.refreshTable)

    def addPortfolio(self, exchange, market):
        """Create new trade in portfolio"""
        d = DialogTrade(self)
        d.loadNewTradeData(exchange, market)
        if d.exec_():
            d.createTrade()
            self.refreshTable()

    def contextoMenuEvent(self, position):
        """Load contextual menu"""
        contextMenu = CustomContextMenu(self.tabla)
        index = self.tabla.indexAt(position)
        if index.isValid():
            contextMenu.handler(position)

    def openDialogEdit(self, trade_id):
        """Edit trade from trade_id"""
        d = DialogTrade(self)
        d.loadEditTradeData(trade_id)
        if d.exec_():
            d.updateTrade()
            self.refreshTable()

    def deleteTrade(self, trade_id):
        """Delete trade from trade_id"""
        result = QtWidgets.QMessageBox.question(
            self, f"Trade {trade_id}", self.tr("Do you want delete this trade?"),
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
        if result == QtWidgets.QMessageBox.Yes:
            Trades.delete_by_id(trade_id)
            self.refreshTable()

    def closeTrade(self, trade_id):
        trade = Trades.get_by_id(trade_id)
        d = QtWidgets.QInputDialog(self)
        d.setInputMode(QtWidgets.QInputDialog.DoubleInput)
        d.setLocale(QLocale('C'))
        d.setWindowTitle(f"Trade {trade_id}")
        d.setLabelText(self.tr("Input close price:"))
        d.setDoubleMaximum(trade.market.last_price * 100)
        d.setDoubleStep(trade.market.last_price * 0.005)
        d.setDoubleDecimals(8)
        d.setDoubleValue(trade.market.last_price)
        if d.exec_():
            trade.close_price = d.doubleValue()
            trade.save()
        self.refreshTable()

    def refreshTable(self):
        """Add data from database"""
        selected = self.combo_trades.currentText().lower()
        if selected == "open":
            data, headers = Trades.getOpenTradesDataAndHeader()
        elif selected == "closed":
            data, headers = Trades.getClosedTradesDataAndHeader()
        else:
            data = []
            headers = []
        self.model = TradesTableModel(data, headers)
        self.tabla.setModel(self.model)
        self.resizeTableColumns()
        self.ed_profit100_total.setText(self.model.suma_profit100)

    def resizeTableColumns(self):
        """Custom resize columns method"""
        self.tabla.resizeColumnsToContents()
        for col, header in enumerate(self.model.headers):
            new_size = self.tabla.columnWidth(col) + 15
            self.tabla.setColumnWidth(col, new_size)

# ────────────────────────────────────────────────────────────────────────────────



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/dock_portfolio.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockPortfolio(object):
    def setupUi(self, DockPortfolio):
        DockPortfolio.setObjectName("DockPortfolio")
        DockPortfolio.resize(405, 325)
        DockPortfolio.setAutoFillBackground(False)
        DockPortfolio.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        DockPortfolio.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        DockPortfolio.setAllowedAreas(QtCore.Qt.BottomDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabla = QtWidgets.QTableView(self.dockWidgetContents)
        self.tabla.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tabla.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tabla.setAlternatingRowColors(True)
        self.tabla.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tabla.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tabla.setGridStyle(QtCore.Qt.SolidLine)
        self.tabla.setObjectName("tabla")
        self.tabla.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.tabla)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.combo_trades = QtWidgets.QComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo_trades.sizePolicy().hasHeightForWidth())
        self.combo_trades.setSizePolicy(sizePolicy)
        self.combo_trades.setMaximumSize(QtCore.QSize(200, 16777215))
        self.combo_trades.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.combo_trades.setInsertPolicy(QtWidgets.QComboBox.InsertAtTop)
        self.combo_trades.setObjectName("combo_trades")
        self.combo_trades.addItem("")
        self.combo_trades.addItem("")
        self.verticalLayout_2.addWidget(self.combo_trades)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.ed_profit100_total = QtWidgets.QLineEdit(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ed_profit100_total.sizePolicy().hasHeightForWidth())
        self.ed_profit100_total.setSizePolicy(sizePolicy)
        self.ed_profit100_total.setMaximumSize(QtCore.QSize(100, 16777215))
        self.ed_profit100_total.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_profit100_total.setReadOnly(True)
        self.ed_profit100_total.setCursorMoveStyle(QtCore.Qt.LogicalMoveStyle)
        self.ed_profit100_total.setObjectName("ed_profit100_total")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.ed_profit100_total)
        self.verticalLayout_2.addLayout(self.formLayout)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout)
        DockPortfolio.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockPortfolio)
        QtCore.QMetaObject.connectSlotsByName(DockPortfolio)

    def retranslateUi(self, DockPortfolio):
        _translate = QtCore.QCoreApplication.translate
        DockPortfolio.setWindowTitle(_translate("DockPortfolio", "Po&rtfolio"))
        self.combo_trades.setItemText(0, _translate("DockPortfolio", "Open"))
        self.combo_trades.setItemText(1, _translate("DockPortfolio", "Closed"))
        self.label.setText(_translate("DockPortfolio", "Profit total:"))
import iconos_rc



from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import Qt, QLocale

from qtradingview.ui.dialog_trade_Ui import Ui_DialogTrade
from qtradingview.models.markets import Markets
from .models import Trades


# ─── DIALOG TRADE ───────────────────────────────────────────────────────────────

class DialogTrade(QDialog, Ui_DialogTrade):
    
    def __init__(self, parent=None, flags=Qt.WindowFlags()):
        QDialog.__init__(self, parent=parent, flags=flags)
        self.setupUi(self)
        #
        self.spin_price.setLocale(QLocale('C'))
        self.spin_amount.setLocale(QLocale('C'))

    def loadNewTradeData(self, exchange, market):
        self.ed_exchange.setText(exchange.title())
        self.ed_market.setText(market.upper())
        self.m = Markets.get_symbol_by_exchange(self.ed_market.text(), self.ed_exchange.text())
        self.spin_price.setValue(self.m.last_price)
        self.spin_price.setSingleStep(self.m.last_price * 0.01)

    def loadEditTradeData(self, trade_id):
        self.t = Trades.get_by_id(trade_id)
        self.ed_exchange.setText(self.t.market.exchange.title())
        self.ed_market.setText(self.t.market.symbol.upper())
        index = self.combo_type.findText(self.t.position.title())
        self.combo_type.setCurrentIndex(index)
        self.spin_price.setValue(self.t.open_price)
        self.spin_price.setSingleStep(self.t.open_price * 0.01)
        self.spin_amount.setValue(self.t.amount)

    def createTrade(self):
        t = Trades(
            market=self.m, position=self.combo_type.currentText().lower(),
            open_price=self.spin_price.value(), amount=self.spin_amount.value()
        )
        t.save()

    def updateTrade(self):
        self.t.position = self.combo_type.currentText().lower()
        self.t.open_price = self.spin_price.value()
        self.t.amount = self.spin_amount.value()
        self.t.save()
        
# ────────────────────────────────────────────────────────────────────────────────



import logging
from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtGui import QIcon, QColor
from PyQt5.QtWidgets import QMenu, QAction

from .models import Alarms


# ─── ALARMS TABLE MODEL ─────────────────────────────────────────────────────────

class AlarmsTableModel(QAbstractTableModel):

    def __init__(self, data, headers):
        QAbstractTableModel.__init__(self)
        self._data = data
        self.headers = headers

    def data(self, index, role=Qt.DisplayRole):
        valor = self._data[index.row()][index.column()]
        col = self.headers[index.column()]
        if role == Qt.DisplayRole:
            if col == "exchange":
                return valor.title()
            return valor
        if role == Qt.BackgroundRole:
            isEnabled = self._data[index.row()][5]
            if isEnabled:
                return QColor(127, 212, 150)
            else:
                return QColor(189, 121, 100)
        if role == Qt.DecorationRole:
            if col == "exchange":
                return QIcon(f":/exchanges/{valor}")

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
            return self.headers[section]
        return QAbstractTableModel.headerData(self, section, orientation, role)

    def rowCount(self, index):
        return len(self._data)

    def columnCount(self, index):
        return len(self.headers)

# ────────────────────────────────────────────────────────────────────────────────


class CustomContextMenu(QMenu):
    
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.mw = self.parent().parent().parent().parent()
        self.dock = self.parent().parent().parent()

    def handler(self, position):
        index = self.parent().indexAt(position)
        self._valor = self.parent().model().data(index)
        self._alarm = self._get_alarm(position)
        if self._valor is not None and self._alarm is not None:
            self._insertActions()
            self.exec_(self.parent().viewport().mapToGlobal(position))

    # ─── PRIVATE METHODS ────────────────────────────────────────────────────────────

    def _insertActions(self):
        if self._alarm.enabled:
            self.addAction(self._action("Disable", self.runActionDisable))
        else:
            self.addAction(self._action("Enable", self.runActionEnable))
        self.addSeparator()
        self.addAction(self._action("Edit...", self.runActionEdit))
        self.addAction(self._action("Delete", self.runActionDelete))

    def _get_alarm(self, position):
        try:
            index = self.parent().indexAt(position)
            newindex = self.parent().model().index(index.row(), 0)
            alarm_id = self.parent().model().data(newindex)
            return Alarms.get_by_id(alarm_id)
        except Exception as e:
            logging.warning("Exception trying get item on this position.")
            logging.error(e.__str__())
            return None

    def _action(self, texto, cmd, icono=None, shortcut=None):
        action = QAction(texto, self, triggered=cmd)
        if shortcut is not None:
            action.setShortcut(shortcut)
        if icono is not None:
            action.setIcon(QIcon(icono))
        return action

    # ─── ACTIONS ────────────────────────────────────────────────────────────────────

    def runActionDisable(self):
        self._alarm.enabled = False
        self._alarm.save()
        self.dock.refreshTable()

    def runActionEnable(self):
        self._alarm.enabled = True
        self._alarm.save()
        self.dock.refreshTable()

    def runActionEdit(self):
        self.dock.editAlarm(self._alarm.id)

    def runActionDelete(self):
        self.dock.deleteAlarm(self._alarm.id)



import logging
from PyQt5 import QtWidgets, QtGui, QtCore

from qtradingview.markets.widgets import CustomItem, CustomItemDelegate, CustomContextMenu
from qtradingview.markets.updater import UpdateAllMarkets

from qtradingview.models.markets import Markets
from qtradingview.ui.dock_markets_Ui import Ui_dock_markets


# ─── DOCK MARKETS ───────────────────────────────────────────────────────────────

class DockMarkets(QtWidgets.QDockWidget, Ui_dock_markets):

    # statusbar_signal = QtCore.pyqtSignal(str)

    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent=parent)
        self.setupUi(self)
        self.setWindowTitle("Markets")
        self.mw = self.parent()  # mainwindow
        self.setVisible(self.mw.actionMarkets.isChecked())
        # base
        self.markets_updater = UpdateAllMarkets(self)
        self.lista_mode = self.mw.cfg.value("markets/list_mode", defaultValue="all")
        self._load_exchanges()
        self._signals()
        # list markets
        self.delegate = CustomItemDelegate()
        self.list_markets.setItemDelegate(self.delegate)
        self.list_markets.customContextMenuRequested.connect(self.contextMenuEvent)
        # markets updater
        self.loadListMode(self.lista_mode)
        self.markets_updater.start()

    def _signals(self):
        self.combo_exchange.currentTextChanged.connect(self.onExchangeChanged)
        self.list_markets.itemDoubleClicked.connect(self.onDoubleClickMarket)
        self.edit_filtro.textEdited.connect(self.onFiltroChanged)
        self.btn_all.toggled.connect(self.onClickAllButton)
        self.btn_favorite.toggled.connect(self.onClickFavoriteButton)
        self.btn_margin.toggled.connect(self.onClickMarginButton)
        self.btn_update.clicked.connect(self.markets_updater.start)
        self.markets_updater.infoEvent.connect(self.onEventUpdater)
        self.markets_updater.onFinished.connect(self.onMarketsUpdaterFinished)
    
    def closeEvent(self, event):
        self.mw.actionMarkets.setChecked(False)
        self.setVisible(False)
        
    @property
    def selected_exchange(self):
        return self.combo_exchange.currentText().lower()

    @property
    def fav_is_checked(self):
        return self.btn_favorite.isChecked()

    def setVisible(self, visible):
        super().setVisible(visible)
        if visible:
            self.raise_()

    # ─── EVENTS ─────────────────────────────────────────────────────────────────────

    def onActionEvent(self, actived):
        """ Show/hide this dock and raise if actived """
        self.setVisible(actived)
        if actived:
            self.raise_()

    def onMarketsUpdaterFinished(self):
        if self.list_markets.count() == 0:
            r = QtWidgets.QMessageBox.information(self, "debes reiniciar", "reiniciaaaa")
            if r:
                self.mw.ctx.app.quit()
        #
        m = Markets.get_symbol_by_exchange("BTC/USDT", "binance")
        self.mw.static_price.setText(f"{m.last_price: .8} BTC/USDT ")
        QtCore.QTimer.singleShot(30 * 1000, self.markets_updater.start)
        self.mw.dock_portfolio.refreshTable()

    # gestiona los shortcuts del dock
    def keyPressEvent(self, event):
        # f5 actualiza markets
        if event.key() == QtCore.Qt.Key_F5:
            self.markets_updater.start()
        # si el foco esta en la lista de markets...
        if self.list_markets.hasFocus():
            item = self.list_markets.currentItem()
            # escape favorite toggle
            if event.key() == QtCore.Qt.Key_Escape:
                item.toggle_favorite()
            # return load selected market
            elif event.key() == QtCore.Qt.Key_Return:
                self.parent().load_chart(item.symbol, item.exchange)
        else:
            logging.debug(event.key())

    def onEventUpdater(self, texto):
        logging.debug(texto)
        self.mw.set_text_status(texto)

    # ─── LIST MODE ──────────────────────────────────────────────────────────────────

    def loadListMode(self, mode):
        if mode == "fav":
            self.btn_favorite.setChecked(True)
            self.onClickFavoriteButton(True)
        elif mode == "margin":
            self.btn_margin.setChecked(True)
            self.onClickMarginButton(True)
        else:
            self.btn_all.setChecked(True)
            self.onClickAllButton(True)

    def onClickAllButton(self, actived):
        self.lista_mode = "all"
        filtro = self.edit_filtro.text()
        for index in range(self.list_markets.count()):
            item = self.list_markets.item(index)
            item.mostrar(self.lista_mode, filtro)

    def onClickFavoriteButton(self, actived):
        self.lista_mode = "fav"
        filtro = self.edit_filtro.text()
        for index in range(self.list_markets.count()):
            item = self.list_markets.item(index)
            item.mostrar(self.lista_mode, filtro)

    def onClickMarginButton(self, actived):
        self.lista_mode = "margin"
        filtro = self.edit_filtro.text()
        for index in range(self.list_markets.count()):
            item = self.list_markets.item(index)
            item.mostrar(self.lista_mode, filtro)

    # ────────────────────────────────────────────────────────────────────────────────

    # filtro de texto
    def onFiltroChanged(self, text):
        """ On write/modify filter field """
        for index in range(self.list_markets.count()):
            item = self.list_markets.item(index)
            item.mostrar(self.lista_mode, text)

    def onExchangeChanged(self):
        """ When exchange combo is changed """
        self.edit_filtro.clear()
        self._load_markets()

    def onDoubleClickMarket(self, item):
        """ When double click over market """
        self.mw.load_chart(item.text(), self.selected_exchange)

    # ─── PRIVATE METHODS ────────────────────────────────────────────────────────────

    def contextMenuEvent(self, position):
        contextMenu = CustomContextMenu(self.list_markets)
        if "QPoint" in str(position):
            contextMenu.handler(position)

    # carga lista de exchanges en el combo
    def _load_exchanges(self):
        self.combo_exchange.blockSignals(True)
        self.combo_exchange.clear()
        for x in self.mw.cfg.value('settings/exchanges'):
            path = f":/exchanges/{x.lower()}"
            self.combo_exchange.addItem(QtGui.QIcon(path), x.title())
        default_index = self.combo_exchange.findText(self.mw.cfg.value('settings/initial_exchange').title())
        if default_index != -1:
            self.combo_exchange.setCurrentIndex(default_index)
        self.combo_exchange.blockSignals(False)
        self.onExchangeChanged()

    # carga lista de markets
    def _load_markets(self):
        self.combo_exchange.setDisabled(True)
        filtro = self.edit_filtro.text()
        self.list_markets.clear()
        for i, item in enumerate(Markets.get_all_by_exchange(self.selected_exchange)):
            if i % 20 == 0:
                QtCore.QCoreApplication.processEvents()
            nuevo = CustomItem(self.list_markets)
            nuevo.configurar(item.symbol, self.selected_exchange)
            nuevo.mostrar(self.lista_mode, filtro)
        self.combo_exchange.setEnabled(True)



import logging

from PyQt5.QtWidgets import QStyledItemDelegate, QListWidgetItem, QStyleOptionViewItem, QMenu, QAction
from PyQt5.QtGui import QIcon

from qtradingview.models.markets import Markets


class CustomItemDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        option.decorationPosition = QStyleOptionViewItem.Right
        super(CustomItemDelegate, self).paint(painter, option, index)


class CustomContextMenu(QMenu):

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.mw = self.parent().parent().parent().parent()  # ref a mainwindow

    def handler(self, position):
        self._item = self._get_item(position)
        if self._item is not None:
            self._insert_favorite(self._item)
            self._insert_actions(self._item)
            self.exec_(self.parent().mapToGlobal(position))

    # ─── PRIVATE METHODS ────────────────────────────────────────────────────────────
    # devuelve el item seleccionado o devuelve None si no lo hay
    def _get_item(self, position):
        try:
            return self.parent().itemAt(position)
        except Exception as e:
            logging.warning("Exception trying get item on this position.")
            logging.error(e.__str__())
            return None

    # funcion para configurar las opciones rapidamente
    def _action(self, texto, cmd, icono=None, shortcut=None):
        action = QAction(texto, self, triggered=cmd)
        if shortcut is not None:
            action.setShortcut(shortcut)
        if icono is not None:
            action.setIcon(QIcon(icono))
        return action

    # inserta los menus que ejecutan eventos
    def _insert_actions(self, item):
        self.addAction(self._action(self.tr("Load chart..."), self._run_load_chart, icono=":/actions/markets", shortcut="Return"))
        self.addSeparator()
        self.addAction(self._action(self.tr("Add to portfolio..."), self._run_add_portfolio, icono=":/actions/portfolio"))
        self.addAction(self._action(self.tr("New alarm..."), self._runNewAlarm, icono=":/actions/alarms"))
        self.addSeparator()
        self.addAction(self._action(self.tr("Set as initial market"), self._run_set_initial_market, icono=":/actions/settings"))

    # inserta menu favorito, ejecuta evento externo
    def _insert_favorite(self, item):
        v = {}
        v[True] = {"txt": self.tr("Remove from favorite"), "ico": ":base/voidstar"}
        v[False] = {"txt": self.tr("Add to favorite"), "ico": ":base/star"}
        self.addAction(self._action(v[item.is_favorite]['txt'], item.toggle_favorite, shortcut="Esc", icono=v[item.is_favorite]['ico']))

    # ─── EVENTS ────────────────────────────────────────────────────────────────────

    def _runNewAlarm(self):
        self.mw.dock_alarms.addAlarm(self._item.exchange, self._item.symbol)

    def _run_load_chart(self):
        self.mw.load_chart(self._item.symbol, self._item.exchange)

    def _run_set_initial_market(self):
        self.mw.config['initial_exchange'] = self._item.exchange.title()
        self.mw.config['initial_market'] = self._item.symbol
        self.mw.ctx.save_config()

    def _run_add_portfolio(self):
        self.mw.dock_portfolio.addPortfolio(self._item.exchange, self._item.symbol)

# ────────────────────────────────────────────────────────────────────────────────

# ─── ITEMS ──────────────────────────────────────────────────────────────────────


class CustomItem(QListWidgetItem):

    exchange = None

    @property
    def symbol(self):
        return self.text()

    @property
    def is_favorite(self):
        return Markets.check_symbol_is_fav(self.symbol, self.exchange)

    @property
    def is_margin(self):
        return Markets.check_symbol_is_margin(self.symbol, self.exchange)

    def _get_icon(self):
        if self.is_favorite:
            return QIcon(":/base/star")
        return QIcon(":/base/voidstar")

    def _mostrar(self):
        self.setHidden(False)

    def _ocultar(self):
        self.setHidden(True)

    def configurar(self, symbol, exchange):
        self.setText(symbol)
        self.exchange = exchange.lower()
        self.setIcon(self._get_icon())

    def mostrar(self, lista_mode, filtro):
        if lista_mode == "all":
            self.mostrar_all(True, filtro)
        elif lista_mode == "fav":
            self.mostrar_fav(True, filtro)
        elif lista_mode == "margin":
            self.mostrar_margin(True, filtro)

    #
    def mostrar_all(self, all_actived, filtro):
        if all_actived:
            if filtro.upper() in self.symbol:
                self._mostrar()
            else:
                self._ocultar()

    def mostrar_fav(self, fav_actived, filtro):
        if fav_actived:
            if self.is_favorite:
                if filtro.upper() in self.symbol:
                    self._mostrar()
                else:
                    self._ocultar()
            else:
                self._ocultar()

    def mostrar_margin(self, margin_actived, filtro):
        if margin_actived:
            if self.is_margin:
                self._mostrar()
            else:
                self._ocultar()
        else:
            self._mostrar()

    def toggle_favorite(self):
        item = Markets.get_symbol_by_exchange(self.text(), self.exchange)
        item.toggle_fav()
        self.setIcon(self._get_icon())

# ────────────────────────────────────────────────────────────────────────────────



from PyQt5.QtWidgets import QDockWidget, QMessageBox
from PyQt5.QtCore import QTimer

from qtradingview.ui.dock_alarms_Ui import Ui_DockAlarms

from .widgets import AlarmsTableModel, CustomContextMenu
from .models import Alarms
from .dialog import DialogAlarm


# ─── DOCK PORTFOLIO ─────────────────────────────────────────────────────────────

class DockAlarms(QDockWidget, Ui_DockAlarms):

    def __init__(self, parent=None):
        QDockWidget.__init__(self, parent=parent)
        self.setupUi(self)
        self.mw = parent
        self.setWindowTitle(self.tr("Alarms"))
        #
        self.setVisible(self.mw.actionAlarms.isChecked())
        self.refreshTable()
        #
        self.alarm_checker = QTimer(self)
        self.alarm_checker.setInterval(10000)
        self.alarm_checker.timeout.connect(self.checkAlarms)
        self.alarm_checker.start()
        self.tb_alarms.customContextMenuRequested.connect(self.contextoMenuEvent)

    def contextoMenuEvent(self, position):
        """Load contextual menu"""
        contextMenu = CustomContextMenu(self.tb_alarms)
        index = self.tb_alarms.indexAt(position)
        if index.isValid():
            contextMenu.handler(position)

    def checkAlarms(self):
        for alarm in Alarms.get_all():
            if alarm.enabled:
                if alarm.isComplished:
                    titulo = f"#{alarm.id} {alarm.market.symbol}"
                    msg = f"Alarm #{alarm.id} | This alarm is complished with condition '{alarm.condition_label} {alarm.price:.8}'"
                    self.mw.notify(titulo, msg)
                    alarm.disable()
                    if alarm.autodelete:
                        alarm.delete_instance()
        self.refreshTable()

    def onActionEvent(self, actived):
        """ Show/hide this dock and raise if actived """
        self.setVisible(actived)
        if actived:
            self.raise_()

    def closeEvent(self, event):
        self.mw.actionAlarms.setChecked(False)
        self.setVisible(False)

    def setVisible(self, active):
        """Show/hide this dock and start/stop refreshing table timer"""
        return super().setVisible(active)

    def refreshTable(self):
        """Add data from database"""
        data, headers = Alarms.getAlarmsToTable()
        self.model = AlarmsTableModel(data, headers)
        self.tb_alarms.setModel(self.model)
        self.tb_alarms.hideColumn(5)
        self.tb_alarms.resizeColumnsToContents()

    def addAlarm(self, exchange, market):
        """ Open dialog to create new alarm"""
        d = DialogAlarm(self)
        d.loadNewAlarm(exchange, market)
        if d.exec_():
            d.createAlarm()
            self.refreshTable()

    def editAlarm(self, alarm_id):
        """ Open dialog to edit alarm"""
        d = DialogAlarm(self)
        d.loadAlarm(alarm_id)
        if d.exec_():
            d.updateAlarm(alarm_id)
            self.refreshTable()

    def deleteAlarm(self, alarm_id):
        """Delete trade from trade_id"""
        result = QMessageBox.question(
            self, f"Alarm {alarm_id}", self.tr("Do you want delete this alarm?"),
            QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if result == QMessageBox.Yes:
            Alarms.delete_by_id(alarm_id)
            self.refreshTable()

# ────────────────────────────────────────────────────────────────────────────────



import logging
from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtGui import QColor, QIcon
from PyQt5.QtWidgets import QMenu, QAction

from .models import Trades


# ─── TRADES MODEL FOR TABLE ─────────────────────────────────────────────────────

class TradesTableModel(QAbstractTableModel):

    def __init__(self, data, headers):
        QAbstractTableModel.__init__(self)
        self._data = data
        self.headers = headers

    @property
    def suma_profit100(self):
        index = self.headers.index('Profit100')
        total = 0.0
        for d in self._data:
            total += d[index]
        return f"{total:.2f}%"

    def data(self, index, role=Qt.DisplayRole):
        """ Insert data  on table """
        col = self.headers[index.column()].lower()
        valor = self._data[index.row()][index.column()]

        if role == Qt.DisplayRole:
            return self.customDisplayRole(col, valor)

        elif role == Qt.ForegroundRole:
            return self.customForegroundRole(col, valor)

        elif role == Qt.TextAlignmentRole:
            return Qt.AlignCenter

        elif role == Qt.DecorationRole:
            if col == "exchange":
                return QIcon(f":/exchanges/{valor}")

        elif role == Qt.ToolTipRole:
            return self.customTooltipRole(col, valor)

    def customDisplayRole(self, col, valor):
        """ Custom method to display values """
        floats_heads = ["openprice", "closeprice", "amount", "lastprice", "profit"]
        if valor is not None:
            if col == "exchange" or col == "postype":
                return valor.title()
            elif col in floats_heads:
                return f"{valor:.8f}"
            elif col == "profit100":
                return f"{valor:.2f}%"
            elif col == "lastupdate":
                return f"{valor[0]}m {valor[1]}s ago"
        return valor

    def customForegroundRole(self, col, valor):
        """ Custom method to set text color """
        if col == "profit100" or col == "profit":
            if valor > 0:
                return QColor('green')
            elif valor < 0:
                return QColor('red')
        elif col == "lastupdate":
            if valor[0] > 2:
                return QColor('red')
        return

    def customTooltipRole(self, col, valor):
        if col == "id":
            return self.tr("Identifier number of this position")
        elif col == "exchange":
            return self.tr("Exchange where be open position")
        elif col == "market":
            return self.tr("Market of this position")
        return
    
    # ────────────────────────────────────────────────────────────────────────────────

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
            return self.headers[section]
        return QAbstractTableModel.headerData(self, section, orientation, role)

    def rowCount(self, index):
        return len(self._data)

    def columnCount(self, index):
        return len(self.headers)

# ────────────────────────────────────────────────────────────────────────────────


# ─── PORTFOLIO CONTEXT MENU ─────────────────────────────────────────────────────

class CustomContextMenu(QMenu):

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.mw = self.parent().parent().parent().parent()
        self.dock = self.parent().parent().parent()

    def handler(self, position):
        index = self.parent().indexAt(position)
        self._valor = self.parent().model().data(index)
        self._trade = self._get_trade(position)
        if self._valor is not None and self._trade is not None:
            self._insertActions()
            self.exec_(self.parent().viewport().mapToGlobal(position))

    # ─── PRIVATE METHODS ────────────────────────────────────────────────────────────

    def _insertActions(self):
        self.addAction(self._action("Close...", self.runActionClose))
        self.addAction(self._action("Edit...", self.runActionEdit))
        self.addAction(self._action("Delete", self.runActionDelete))
        self.addSeparator()
        self.addAction(self._action(f"Copy to clipboard: '{self._valor}'", self.runActionCopyClipboard))

    def _get_trade(self, position):
        try:
            index = self.parent().indexAt(position)
            newindex = self.parent().model().index(index.row(), 0)
            trade_id = self.parent().model().data(newindex)
            return Trades.get_by_id(trade_id)
        except Exception as e:
            logging.warning("Exception trying get item on this position.")
            logging.error(e.__str__())
            return None

    def _action(self, texto, cmd, icono=None, shortcut=None):
        action = QAction(texto, self, triggered=cmd)
        if shortcut is not None:
            action.setShortcut(shortcut)
        if icono is not None:
            action.setIcon(QIcon(icono))
        return action

    # ─── ACTIONS ────────────────────────────────────────────────────────────────────

    def runActionClose(self):
        self.dock.closeTrade(self._trade.id)

    def runActionEdit(self):
        self.dock.openDialogEdit(self._trade.id)

    def runActionDelete(self):
        self.dock.deleteTrade(self._trade.id)

    def runActionCopyClipboard(self):
        clip = self.mw.ctx.app.clipboard()
        clip.setText(self._valor)

# ────────────────────────────────────────────────────────────────────────────────



import peewee
from qtradingview.models.base import CustomModel
from qtradingview.models.markets import Markets


# ─── TRADE MODEL ────────────────────────────────────────────────────────────────

class Trades(CustomModel):

    market = peewee.ForeignKeyField(Markets, backref="trades")
    open_price = peewee.FloatField()
    close_price = peewee.FloatField(null=True)
    position = peewee.CharField(max_length=5)
    amount = peewee.FloatField()

    @classmethod
    def get_by_id(cls, id):
        return cls.get(cls.id == id)

    @classmethod
    def get_all(cls):
        return cls.select()
    
    @classmethod
    def getOpenTradesDataAndHeader(cls):
        data = []
        headers = [
            'ID', 'Exchange', 'Market', 'PosType', 'OpenPrice',
            'Amount', 'LastPrice', 'LastUpdate', 'Profit100', 'Profit'
        ]
        for t in cls.select().where(cls.close_price.is_null(True)):
            data.append([
                t.id, t.market.exchange, t.market.symbol, t.position,
                t.open_price, t.amount, t.market.last_price, t.market.since_update,
                t.profit100, t.profit
            ])
        return data, headers
    
    @classmethod
    def getClosedTradesDataAndHeader(cls):
        data = []
        headers = [
            'ID', 'Exchange', 'Market', 'PosType', 'OpenPrice',
            'Amount', 'ClosePrice', 'Profit100', 'Profit'
        ]
        for t in cls.select().where(cls.close_price.is_null(False)):
            data.append([
                t.id, t.market.exchange, t.market.symbol, t.position,
                t.open_price, t.amount, t.close_price,
                t.profit100, t.profit
            ])
        return data, headers
    
    @property
    def isClosed(self):
        if self.close_price is not None:
            return True
        return False

    def getClosePrice(self):
        if self.isClosed:
            return self.close_price
        return self.market.last_price

    @property
    def profit(self):
        close_price = self.getClosePrice()
        if close_price is not None:
            close_total = close_price * self.amount
            open_total = self.open_price * self.amount
            if self.position == "long":
                return close_total - open_total
            if self.position == "short":
                return open_total - close_total
        return None

    @property
    def profit100(self):
        close_price = self.getClosePrice()
        if close_price is not None:
            if self.position == "long":
                return ((close_price - self.open_price) / self.open_price) * 100
            if self.position == "short":
                return ((self.open_price - close_price) / close_price) * 100
        return None

# ────────────────────────────────────────────────────────────────────────────────



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.ApplicationModal)
        MainWindow.resize(673, 480)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(600, 480))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/base/logo"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet("QMainWindow::separator{width: 4px}")
        MainWindow.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        MainWindow.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AllowTabbedDocks|QtWidgets.QMainWindow.AnimatedDocks|QtWidgets.QMainWindow.ForceTabbedDocks)
        MainWindow.setProperty("currentExchange", "")
        MainWindow.setProperty("currentMarket", "")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.webview = QtWebEngineWidgets.QWebEngineView(self.centralwidget)
        self.webview.setObjectName("webview")
        self.horizontalLayout.addWidget(self.webview)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.tradingbar = QtWidgets.QToolBar(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tradingbar.sizePolicy().hasHeightForWidth())
        self.tradingbar.setSizePolicy(sizePolicy)
        self.tradingbar.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.tradingbar.setOrientation(QtCore.Qt.Horizontal)
        self.tradingbar.setIconSize(QtCore.QSize(24, 24))
        self.tradingbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.tradingbar.setObjectName("tradingbar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.tradingbar)
        self.appbar = QtWidgets.QToolBar(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.appbar.sizePolicy().hasHeightForWidth())
        self.appbar.setSizePolicy(sizePolicy)
        self.appbar.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.appbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.appbar.setObjectName("appbar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.appbar)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 673, 20))
        self.menuBar.setDefaultUp(False)
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtWidgets.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menuBar)
        self.menuView.setObjectName("menuView")
        self.menuHelp = QtWidgets.QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menuBar)
        self.actionSettings = QtWidgets.QAction(MainWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/actions/settings"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSettings.setIcon(icon1)
        self.actionSettings.setObjectName("actionSettings")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/actions/exit"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionQuit.setIcon(icon2)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setIcon(icon)
        self.actionAbout.setObjectName("actionAbout")
        self.actionMarkets = QtWidgets.QAction(MainWindow)
        self.actionMarkets.setCheckable(True)
        self.actionMarkets.setChecked(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/actions/markets"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMarkets.setIcon(icon3)
        self.actionMarkets.setObjectName("actionMarkets")
        self.actionAlarms = QtWidgets.QAction(MainWindow)
        self.actionAlarms.setCheckable(True)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/actions/alarms"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAlarms.setIcon(icon4)
        self.actionAlarms.setObjectName("actionAlarms")
        self.actionFull_Screen = QtWidgets.QAction(MainWindow)
        self.actionFull_Screen.setCheckable(True)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/actions/screen"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFull_Screen.setIcon(icon5)
        self.actionFull_Screen.setObjectName("actionFull_Screen")
        self.actionDebug = QtWidgets.QAction(MainWindow)
        self.actionDebug.setCheckable(True)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/actions/debug"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionDebug.setIcon(icon6)
        self.actionDebug.setObjectName("actionDebug")
        self.actionToolbar = QtWidgets.QAction(MainWindow)
        self.actionToolbar.setCheckable(True)
        self.actionToolbar.setChecked(True)
        self.actionToolbar.setObjectName("actionToolbar")
        self.actionPortfolio = QtWidgets.QAction(MainWindow)
        self.actionPortfolio.setCheckable(True)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/actions/portfolio"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPortfolio.setIcon(icon7)
        self.actionPortfolio.setShortcutContext(QtCore.Qt.WindowShortcut)
        self.actionPortfolio.setObjectName("actionPortfolio")
        self.tradingbar.addAction(self.actionMarkets)
        self.tradingbar.addAction(self.actionAlarms)
        self.tradingbar.addAction(self.actionPortfolio)
        self.appbar.addAction(self.actionQuit)
        self.appbar.addAction(self.actionSettings)
        self.appbar.addSeparator()
        self.appbar.addAction(self.actionDebug)
        self.appbar.addSeparator()
        self.appbar.addAction(self.actionFull_Screen)
        self.menuFile.addAction(self.actionSettings)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuView.addAction(self.actionToolbar)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionMarkets)
        self.menuView.addAction(self.actionAlarms)
        self.menuView.addAction(self.actionPortfolio)
        self.menuView.addAction(self.actionDebug)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionFull_Screen)
        self.menuHelp.addAction(self.actionAbout)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menuView.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.actionToolbar.toggled['bool'].connect(self.appbar.setVisible)
        self.actionToolbar.toggled['bool'].connect(self.tradingbar.setVisible)
        self.actionQuit.triggered.connect(MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "QTradingview"))
        self.tradingbar.setWindowTitle(_translate("MainWindow", "TradingBar"))
        self.appbar.setWindowTitle(_translate("MainWindow", "AppBar"))
        self.menuFile.setTitle(_translate("MainWindow", "Fi&le"))
        self.menuView.setTitle(_translate("MainWindow", "Vie&w"))
        self.menuHelp.setTitle(_translate("MainWindow", "&Help"))
        self.actionSettings.setText(_translate("MainWindow", "Settings"))
        self.actionSettings.setToolTip(_translate("MainWindow", "Open settings window"))
        self.actionSettings.setShortcut(_translate("MainWindow", "F9"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionQuit.setToolTip(_translate("MainWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionMarkets.setText(_translate("MainWindow", "Markets"))
        self.actionMarkets.setToolTip(_translate("MainWindow", "Show/hide markets"))
        self.actionMarkets.setShortcut(_translate("MainWindow", "F1"))
        self.actionAlarms.setText(_translate("MainWindow", "Alarms"))
        self.actionAlarms.setToolTip(_translate("MainWindow", "Show/hide alarms"))
        self.actionAlarms.setShortcut(_translate("MainWindow", "F2"))
        self.actionFull_Screen.setText(_translate("MainWindow", "Full Screen"))
        self.actionFull_Screen.setToolTip(_translate("MainWindow", "Activate/deactivate full screen"))
        self.actionFull_Screen.setShortcut(_translate("MainWindow", "F11"))
        self.actionDebug.setText(_translate("MainWindow", "Debug"))
        self.actionDebug.setToolTip(_translate("MainWindow", "Show/hide debug panel"))
        self.actionToolbar.setText(_translate("MainWindow", "Toolbar"))
        self.actionToolbar.setStatusTip(_translate("MainWindow", "Show/hide toolbar"))
        self.actionToolbar.setShortcut(_translate("MainWindow", "F4"))
        self.actionPortfolio.setText(_translate("MainWindow", "Portfolio"))
        self.actionPortfolio.setShortcut(_translate("MainWindow", "F3"))
from PyQt5 import QtWebEngineWidgets
import iconos_rc



from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import QLocale

from qtradingview.ui.dialog_alarm_Ui import Ui_dialogAlarm
from qtradingview.models.markets import Markets
from .models import Alarms


# ─── DIALOG ALARMS ──────────────────────────────────────────────────────────────

class DialogAlarm(QDialog, Ui_dialogAlarm):

    def __init__(self, parent=None):
        QDialog.__init__(self, parent=parent)
        self.setupUi(self)
        #
        for c in Alarms.CONDITIONS_CHOICES:
            self.cb_condition.addItem(c[1], c[0])
        self.spin_price.setLocale(QLocale('C'))

    def loadNewAlarm(self, exchange, market):
        """ Load data to create new alarm """
        self.ed_exchange.setText(exchange.title())
        self.ed_market.setText(market.upper())
        self.m = Markets.get_symbol_by_exchange(self.ed_market.text(), self.ed_exchange.text())
        self.ed_lastPrice.setText(f"{self.m.last_price:.8}")
        self.spin_price.setMaximum(self.m.last_price * 100)
        self.spin_price.setSingleStep(self.m.last_price * 0.005)
        self.spin_price.setValue(self.m.last_price)

    def loadAlarm(self, alarm_id):
        """ Load data from alarm """
        alarm = Alarms.get_by_id(alarm_id)
        self.ed_exchange.setText(alarm.market.exchange)
        self.ed_market.setText(alarm.market.symbol)
        self.ed_lastPrice.setText(f"{alarm.market.last_price:.8}")
        self.cb_condition.setCurrentIndex(alarm.condition)
        self.ck_autodelete.setChecked(alarm.autodelete)
        self.spin_price.setMaximum(alarm.market.last_price * 100)
        self.spin_price.setSingleStep(alarm.market.last_price * 0.005)
        self.spin_price.setValue(alarm.price)

    def createAlarm(self):
        """ Save data to new alarm """
        al = Alarms(
            market=self.m, condition=self.cb_condition.currentData(),
            price=self.spin_price.value(), autodelete=self.ck_autodelete.isChecked()
        )
        al.save()

    def updateAlarm(self, alarm_id):
        """ Update data of alarm """
        alarm = Alarms.get_by_id(alarm_id)
        alarm.condition = self.cb_condition.currentData()
        alarm.price = self.spin_price.value()
        alarm.autodelete = self.ck_autodelete.isChecked()
        alarm.save()

# ────────────────────────────────────────────────────────────────────────────────



import peewee
from qtradingview.models.base import CustomModel
from qtradingview.models.markets import Markets


# ─── TRADE MODEL ────────────────────────────────────────────────────────────────


class Alarms(CustomModel):
    CONDITIONS_CHOICES = (
        (0, "Less than"),
        (1, "Great than")
    )
    market = peewee.ForeignKeyField(column_name='market', model=Markets, null=True)
    description = peewee.CharField(null=True)
    condition = peewee.IntegerField(choices=CONDITIONS_CHOICES, default=0)
    price = peewee.FloatField(null=True)
    enabled = peewee.BooleanField(null=True, default=True)
    autodelete = peewee.BooleanField(null=True)

    @classmethod
    def get_all(cls):
        return cls.select()

    @property
    def condition_label(self):
        return dict(self.CONDITIONS_CHOICES)[self.condition]

    @classmethod
    def getAlarmsToTable(cls):
        data = []
        headers = ['id', 'exchange', 'market', 'condition', 'price', 'enabled']
        for t in cls.select():
            data.append([
                t.id, t.market.exchange, t.market.symbol,
                t.condition_label, t.price, t.enabled
            ])
        return data, headers

    @property
    def isComplished(self):
        if self.condition == 0:
            if self.market.last_price < self.price:
                return True
        if self.condition == 1:
            if self.market.last_price > self.price:
                return True
        return False

    def disable(self):
        self.enabled = False
        self.save()

    def enable(self):
        self.enabled = True
        self.save()
# ────────────────────────────────────────────────────────────────────────────────



import os
import shutil

from signal import signal, SIGINT, SIG_DFL
from cached_property import cached_property

from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QLibraryInfo, QTranslator, QSettings

from qtradingview.utils import AppUtil
from qtradingview.models.base import get_db, migrate

from qtradingview.models.markets import Markets
from qtradingview.portfolio.models import Trades
from qtradingview.alarms.models import Alarms

from qtradingview.base.mainwindow import MainWindow


# ─── CONTEXTO APP ───────────────────────────────────────────────────────────────

class ContextoApp:

    #     os.system("export QT_LOGGING_RULES='*=false'")
    def __init__(self, args):
        self.app = QApplication([])
        self.debug = args['--debug']
        signal(SIGINT, SIG_DFL)
        self.args = args
        self.setDefaultConfig()

    def run(self):
        self.window.show()
        return self.app.exec_()

    def tr(self, context, message):
        return self.app.translate(context, message)

    def deleteDatabaseFile(self):
        dirname = os.path.dirname(self.settings.fileName())
        nou_path = os.path.join(dirname, 'database.db')
        os.remove(nou_path)
        if not AppUtil.isPyinstaller():
            shutil.rmtree('migrations', ignore_errors=True)

    def createDB(self):
        self.db.create_tables([Markets, Trades, Alarms])

    def createTablesDB(self):
        """ Creating or updating database tables. """
        if AppUtil.isPyinstaller():
            self.db.create_tables([Markets, Trades, Alarms])
        else:
            migrate(self.db)

    def checkTablesExists(self):
        """ Returns True if all tables exists """
        if not Markets.table_exists():
            return False
        if not Trades.table_exists():
            return False
        if not Alarms.table_exists():
            return False
        return True

    def setDefaultConfig(self):
        if self.settings.value("settings/exchanges") is None:
            self.settings.setValue("settings/exchanges", ["Binance", "Bitfinex"])
        if self.settings.value("settings/language") is None:
            self.settings.setValue("settings/language", "en_EN")
        if self.settings.value("settings/initial_exchange") is None:
            self.settings.setValue("settings/initial_exchange", "Binance")
        if self.settings.value("settings/initial_market") is None:
            self.settings.setValue("settings/initial_market", "BTC/USDT")

    # ─── PROPIEDADES ────────────────────────────────────────────────────────────────

    @cached_property
    def settings(self):
        return QSettings("QTradingView", "Settings")

    @cached_property
    def window(self):
        return MainWindow(self)

    @cached_property
    def db(self):
        db = get_db()
        return db

    @cached_property
    def app_language(self):
        language = self.settings.value('settings/language')
        qtrans = QTranslator()
        qtrans.load(language, AppUtil.get_i18n_dir())
        return qtrans

    @cached_property
    def system_language(self):
        language = self.settings.value('settings/language')
        qtrans = QTranslator()
        lang = f"qtbase_{language}"
        qtrans.load(lang, QLibraryInfo.location(QLibraryInfo.TranslationsPath))
        return qtrans

# ────────────────────────────────────────────────────────────────────────────────



import os
import sys
import toml
from pathlib import Path


# mandatory to compile with pyinstaller
def resource_path(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.abspath(".")
    return os.path.join(base_path, relative_path)


# ─── APPUTIL ────────────────────────────────────────────────────────────────────

class AppUtil:

    default_config = """
        language = "es_ES"
        exchanges = [ "Bittrex", "Bitfinex", "Binance", "Poloniex",]
        initial_exchange = "Binance"
        initial_market = "BTC/USDT"
        panel_markets_active = false
        panel_portfolio_active = false
        panel_debug_active = false
    """

    @classmethod
    def get_app_dir(cls):
        """ Return app dir into home folder """
        return os.path.join(Path.home(), ".qtradingview")

    @classmethod
    def get_config_file_path(cls):
        """ Return config file path """
        return os.path.join(cls.get_app_dir(), "config.toml")

    @classmethod
    def get_db_file_path(cls):
        """ Return database file path """
        return os.path.join(cls.get_app_dir(), "database.db")

    @classmethod
    def get_i18n_dir(cls):
        """ Return i18n resource path """
        return resource_path(os.path.join("app", "i18n"))

    @classmethod
    def create_app_dir(cls):
        """ Create app folder if not exists """
        if not os.path.exists(cls.get_app_dir()):
            os.mkdir(cls.get_app_dir())

    @classmethod
    def create_default_config(cls):
        """ Create default config file if not exists """
        if not os.path.exists(cls.get_config_file_path()):
            config = toml.loads(cls.default_config)
            with open(AppUtil.get_config_file_path(), "w") as f:
                toml.dump(config, f)

    @classmethod
    def isPyinstaller(cls):
        try:
            _ = sys._MEIPASS
            return True
        except Exception:
            return False
    
    @classmethod
    def existsFileDB(cls):
        return os.path.isfile(cls.get_db_file_path())

# ────────────────────────────────────────────────────────────────────────────────



from PyQt5.QtWidgets import QDialog, QMessageBox

from qtradingview.models.markets import Markets
from qtradingview.ui.dialog_config_Ui import Ui_DialogConfig


# ─── CONFIG DIALOG ──────────────────────────────────────────────────────────────

class DialogConfig(QDialog, Ui_DialogConfig):

    def __init__(self, parent=None, *args, **kwargs):
        QDialog.__init__(self, parent=parent, *args, **kwargs)
        self.setupUi(self)
        self.mw = parent
        #
        self.list_exchanges.sortItems()
        self.cfg = self.mw.ctx.settings
        #
        self.loadConfig()
        self.combo_initial_exchange.currentTextChanged.connect(self.onSelectInitialExchange)
        self.combo_languages.currentTextChanged.connect(self.onChangeLanguage)
        self.onSelectInitialExchange()

    def loadConfig(self):
        # add language
        self.combo_languages.addItem(self.tr("Spanish"), "es_ES")
        self.combo_languages.addItem(self.tr("English"), "en_EN")
        self._select_language(self.cfg.value("settings/language"))
        # exchanges list and initial exchange
        self._select_exchanges(self.cfg.value("settings/exchanges"))
        self._select_initial_exchange()
        # copy exchanges list actived to compare on exit
        self.old_exchanges = self.cfg.value("settings/exchanges")
     
    @property
    def exchanges_is_changed(self):
        """ Return true if exchanges list is modified """
        return self.old_exchanges != self.cfg.value("settings/exchanges")

    # ─── EVENTOS ────────────────────────────────────────────────────────────────────

    def onChangeLanguage(self, language):
        mbox = QMessageBox(self)
        mbox.setIcon(QMessageBox.Information)
        mbox.setWindowTitle(self.tr("Language changed"))
        mbox.setText(self.tr("The language change will be applied when restarting the application"))
        mbox.setStandardButtons(QMessageBox.Ok)
        mbox.show()

    def onSelectInitialExchange(self):
        """ Load initial markets of selected initial exchange """
        self.combo_initial_market.clear()
        for it in Markets.get_all_by_exchange(self.combo_initial_exchange.currentText().lower()):
            self.combo_initial_market.addItem(it.symbol)
        index = self.combo_initial_market.findText(self.cfg.value("settings/initial_market"))
        self.combo_initial_market.setCurrentIndex(index)

    # ─── load methods ───────────────────────────────────────────────────────

    def _select_initial_exchange(self):
        """ Select initial exchange configured in config file """
        index = self.combo_initial_exchange.findText(self.cfg.value("settings/initial_exchange"))
        self.combo_initial_exchange.setCurrentIndex(index)

    def _select_exchanges(self, exchanges):
        """ Select exchanges actived in config file """
        for index in range(self.list_exchanges.count()):
            item = self.list_exchanges.item(index)
            if item.text() in exchanges:
                item.setSelected(True)
            # añade la lista al combo de paso
            self.combo_initial_exchange.addItem(item.text())

    def _select_language(self, language):
        """ Select language defined in config file """
        index = self.combo_languages.findData(language)
        self.combo_languages.setCurrentIndex(index)

    # ─── SAVING-EXIT METHODS ────────────────────────────────────────────────────────

    # devuelve lista de los exchanges seleccionados
    def _get_list_exchanges(self):
        lista = []
        for index in range(self.list_exchanges.count()):
            item = self.list_exchanges.item(index)
            if item.isSelected():
                lista.append(item.text())
        return lista

    # actualiza self.config y guarda cambios al fichero
    def accept(self):
        self.cfg.setValue("settings/language", self.combo_languages.currentData())
        self.cfg.setValue("settings/exchanges", self._get_list_exchanges())
        self.cfg.setValue("settings/initial_exchange", self.combo_initial_exchange.currentText())
        self.cfg.setValue("settings/initial_market", self.combo_initial_market.currentText())
        return super().accept()
# ────────────────────────────────────────────────────────────────────────────────



from PyQt5 import QtWidgets

from qtradingview.ui.about_Ui import Ui_aboutDialog


class DialogAbout(QtWidgets.QDialog, Ui_aboutDialog):

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/dialog_config.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogConfig(object):
    def setupUi(self, DialogConfig):
        DialogConfig.setObjectName("DialogConfig")
        DialogConfig.setWindowModality(QtCore.Qt.WindowModal)
        DialogConfig.resize(250, 463)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DialogConfig.sizePolicy().hasHeightForWidth())
        DialogConfig.setSizePolicy(sizePolicy)
        DialogConfig.setMinimumSize(QtCore.QSize(250, 300))
        DialogConfig.setMaximumSize(QtCore.QSize(250, 800))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/actions/settings"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        DialogConfig.setWindowIcon(icon)
        DialogConfig.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(DialogConfig)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(DialogConfig)
        self.label.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.combo_languages = QtWidgets.QComboBox(DialogConfig)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo_languages.sizePolicy().hasHeightForWidth())
        self.combo_languages.setSizePolicy(sizePolicy)
        self.combo_languages.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.combo_languages.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.combo_languages.setObjectName("combo_languages")
        self.horizontalLayout.addWidget(self.combo_languages)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(DialogConfig)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(DialogConfig)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.combo_initial_market = QtWidgets.QComboBox(DialogConfig)
        self.combo_initial_market.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.combo_initial_market.setObjectName("combo_initial_market")
        self.verticalLayout_2.addWidget(self.combo_initial_market)
        self.combo_initial_exchange = QtWidgets.QComboBox(DialogConfig)
        self.combo_initial_exchange.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.combo_initial_exchange.setObjectName("combo_initial_exchange")
        self.verticalLayout_2.addWidget(self.combo_initial_exchange)
        self.verticalLayout.addLayout(self.verticalLayout_2)
        self.line_2 = QtWidgets.QFrame(DialogConfig)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.list_exchanges = QtWidgets.QListWidget(DialogConfig)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_exchanges.sizePolicy().hasHeightForWidth())
        self.list_exchanges.setSizePolicy(sizePolicy)
        self.list_exchanges.setMinimumSize(QtCore.QSize(0, 150))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.list_exchanges.setFont(font)
        self.list_exchanges.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.list_exchanges.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.list_exchanges.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.list_exchanges.setLineWidth(1)
        self.list_exchanges.setMidLineWidth(0)
        self.list_exchanges.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.list_exchanges.setAlternatingRowColors(True)
        self.list_exchanges.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.list_exchanges.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.list_exchanges.setIconSize(QtCore.QSize(24, 24))
        self.list_exchanges.setMovement(QtWidgets.QListView.Static)
        self.list_exchanges.setFlow(QtWidgets.QListView.TopToBottom)
        self.list_exchanges.setViewMode(QtWidgets.QListView.ListMode)
        self.list_exchanges.setObjectName("list_exchanges")
        item = QtWidgets.QListWidgetItem()
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/exchanges/binance"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon1)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/exchanges/bitbay"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon2)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/exchanges/bitfinex"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon3)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/exchanges/bithumb"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon4)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/exchanges/bitso"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon5)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/exchanges/bitstamp"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon6)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/exchanges/bittrex"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon7)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/exchanges/coinbase"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon8)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/exchanges/gemini"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon9)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/exchanges/hitbtc"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon10)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/exchanges/kraken"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon11)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/exchanges/kucoin"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon12)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/exchanges/okcoin"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon13)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/exchanges/poloniex"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon14)
        self.list_exchanges.addItem(item)
        item = QtWidgets.QListWidgetItem()
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/exchanges/therock"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon15)
        self.list_exchanges.addItem(item)
        self.verticalLayout.addWidget(self.list_exchanges)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogConfig)
        self.buttonBox.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(DialogConfig)
        self.buttonBox.accepted.connect(DialogConfig.accept)
        self.buttonBox.rejected.connect(DialogConfig.reject)
        QtCore.QMetaObject.connectSlotsByName(DialogConfig)

    def retranslateUi(self, DialogConfig):
        _translate = QtCore.QCoreApplication.translate
        DialogConfig.setWindowTitle(_translate("DialogConfig", "Settings"))
        self.label.setText(_translate("DialogConfig", "Language"))
        self.label_2.setText(_translate("DialogConfig", "Initial market"))
        self.list_exchanges.setSortingEnabled(True)
        __sortingEnabled = self.list_exchanges.isSortingEnabled()
        self.list_exchanges.setSortingEnabled(False)
        item = self.list_exchanges.item(0)
        item.setText(_translate("DialogConfig", "Binance"))
        item = self.list_exchanges.item(1)
        item.setText(_translate("DialogConfig", "BitBay"))
        item = self.list_exchanges.item(2)
        item.setText(_translate("DialogConfig", "Bitfinex"))
        item = self.list_exchanges.item(3)
        item.setText(_translate("DialogConfig", "Bithumb"))
        item = self.list_exchanges.item(4)
        item.setText(_translate("DialogConfig", "Bitso"))
        item = self.list_exchanges.item(5)
        item.setText(_translate("DialogConfig", "Bitstamp"))
        item = self.list_exchanges.item(6)
        item.setText(_translate("DialogConfig", "Bittrex"))
        item = self.list_exchanges.item(7)
        item.setText(_translate("DialogConfig", "Coinbase"))
        item = self.list_exchanges.item(8)
        item.setText(_translate("DialogConfig", "Gemini"))
        item = self.list_exchanges.item(9)
        item.setText(_translate("DialogConfig", "HitBTC"))
        item = self.list_exchanges.item(10)
        item.setText(_translate("DialogConfig", "Kraken"))
        item = self.list_exchanges.item(11)
        item.setText(_translate("DialogConfig", "Kucoin"))
        item = self.list_exchanges.item(12)
        item.setText(_translate("DialogConfig", "OKCoin"))
        item = self.list_exchanges.item(13)
        item.setText(_translate("DialogConfig", "Poloniex"))
        item = self.list_exchanges.item(14)
        item.setText(_translate("DialogConfig", "TheRock"))
        self.list_exchanges.setSortingEnabled(__sortingEnabled)
import iconos_rc



"""
QTradingView

Usage:
  qtradingview [--debug]
  qtradingview [--deletedb]
  qtradingview [--updatedb]
  qtradingview -h | --help
  qtradingview -v | --version


Options:
  -h --help     Show this screen.
  -v --version  Show version.
  --debug       Execute in debug mode.
  --updatedb    Update db tables.
  --deletedb    Delete db file.
"""
# export QT_LOGGING_RULES='*=false'
__version__ = '0.17.0'

import sys
import docopt

from qtradingview.context import ContextoApp


def run():
    args = docopt.docopt(__doc__, version=__version__)
    appctx = ContextoApp(args)

    if args['--deletedb']:
        appctx.deleteDatabaseFile()
    appctx.createDB()

    appctx.app.installTranslator(appctx.app_language)
    appctx.app.installTranslator(appctx.system_language)
    exit_code = appctx.run()
    sys.exit(exit_code)


# ─── MAIN ───────────────────────────────────────────────────────────────────────

if __name__ == "__main__":
    run()



import logging
# from bs4 import BeautifulSoup

from PyQt5.QtWidgets import QSplashScreen, QDesktopWidget, QFrame
from PyQt5.QtCore import Qt, QTimer
from PyQt5.QtGui import QPixmap

from PyQt5.QtWebEngineWidgets import QWebEnginePage


# ─── CUSTOM WEBENGINEPAGE ───────────────────────────────────────────────────────

class CustomWebEnginePage(QWebEnginePage):

    adblocker_2020 = """
        (
            function() {
                const checkAd = setInterval(() => {
                    const adBox = document.getElementById('tv-toasts');
                    if (adBox) {
                        adBox.remove();
                        console.log("ad removed");
                    }
                    const adWrapper = document.querySelectorAll("[class^='toast-positioning-wrapper-']")[0];
                    if (adWrapper)  {
                        adWrapper.remove();
                        console.log("ad removed");
                    }
                }, 1500);
            }
        )();"""

    def __init__(self, parent):
        super().__init__(parent)
        self.mw = self.parent().parent().parent()
        #
        self.timer_titulo = QTimer(self)
        self.html = ''
        self.loadFinished.connect(self._on_load_finished)
        self.loadStarted.connect(self._on_load_started)
        self.timer_titulo.timeout.connect(self.update_titulo)

    # al iniciar una carga detiene el timer que elimina la publicidad
    def _on_load_started(self):
        self.clear_titulo()
        self.timer_titulo.stop()

    # al finalizar la carga activa el adblocker y updater titulo
    def _on_load_finished(self):
        self.html = self.toHtml(self._get_html)
        self.timer_titulo.start(3000)
        self.adblocker_tradingview()

    def clear_titulo(self):
        self.mw.setWindowTitle("QTradingView")

    # actualiza el titulo de la ventana con el market y precio actual
    def update_titulo(self):
        self.html = self.toHtml(self._get_html)
        try:
            v = self.title().split(" ")
            titulo = f"{v[0]} {v[1]} {v[2]} | QTradingView"
        except Exception as e:
            logging.debug(f"can't read title, market not load? {e}")
            titulo = "QTradingView"
        finally:
            self.mw.setWindowTitle(titulo)

    # funcion necesaria para ver el codigo html (toHtml)
    def _get_html(self, html_str):
        self.html = html_str
        # if html_str is not None:
        #     soup = BeautifulSoup(html_str, 'html.parser')
        #     for part in soup.select('[class*="toast"]'):
        #         print(part['class'])
        return html_str

    # javascripts
    def javaScriptConsoleMessage(self, level, msg, line, sourceID):
        logging.debug(msg)

    def adblocker_tradingview(self):
        self.runJavaScript(self.adblocker_2020)

# ────────────────────────────────────────────────────────────────────────────────


# ─── SPLASH SCREEN ──────────────────────────────────────────────────────────────

class CustomSplashScreen(QSplashScreen):

    imagen = ":/base/splash"

    def __init__(self, parent=None):
        screen_size = QDesktopWidget().screenGeometry(-1)
        pixmap = QPixmap(self.imagen).scaledToWidth(screen_size.width() / 3)
        super().__init__(parent, pixmap, Qt.SplashScreen)
        self.setMask(pixmap.mask())
        self.set_texto(self.tr("Loading"))
        self.show()

    def set_texto(self, texto, size=3):
        self.showMessage(f"<h{size}>{self.tr(texto)}</h{size}>", Qt.AlignCenter | Qt.AlignBottom)

# ────────────────────────────────────────────────────────────────────────────────


class VLine(QFrame):
    # a simple VLine, like the one you get from designer
    def __init__(self):
        super(VLine, self).__init__()
        self.setFrameShape(self.VLine | self.Sunken)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/dock_alarms.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockAlarms(object):
    def setupUi(self, DockAlarms):
        DockAlarms.setObjectName("DockAlarms")
        DockAlarms.resize(611, 412)
        DockAlarms.setToolTip("")
        DockAlarms.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        DockAlarms.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        DockAlarms.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tb_alarms = QtWidgets.QTableView(self.dockWidgetContents)
        self.tb_alarms.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tb_alarms.setAlternatingRowColors(True)
        self.tb_alarms.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tb_alarms.setObjectName("tb_alarms")
        self.tb_alarms.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tb_alarms)
        self.btn_clear = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btn_clear.setObjectName("btn_clear")
        self.verticalLayout.addWidget(self.btn_clear)
        DockAlarms.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockAlarms)
        QtCore.QMetaObject.connectSlotsByName(DockAlarms)

    def retranslateUi(self, DockAlarms):
        _translate = QtCore.QCoreApplication.translate
        self.btn_clear.setText(_translate("DockAlarms", "Clear"))



import logging
import coloredlogs

from PyQt5 import QtWidgets
from qtradingview.ui.dock_debug_Ui import Ui_DockDebug


# ─── PANEL DEBUG ────────────────────────────────────────────────────────────────

class DockDebug(QtWidgets.QDockWidget, Ui_DockDebug):

    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(self.tr('Debug'))
        #
        self.mw = parent
        self.setVisible(self.mw.actionDebug.isChecked())

    def closeEvent(self, event):
        self.mw.actionDebug.setChecked(False)
        self.setVisible(False)

    def onActionEvent(self, actived):
        """ Show/hide this dock and raise if actived """
        self.setVisible(actived)
        if actived:
            self.raise_()

# ────────────────────────────────────────────────────────────────────────────────


# ─── QLOGGER CLASS ──────────────────────────────────────────────────────────────

class Qlogger(logging.Handler):

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        log_mode = logging.INFO
        if self.parent.ctx.debug:
            log_mode = logging.DEBUG
        coloredlogs.install(level=log_mode)
        self.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s'))

    def emit(self, record):
        msg = self.format(record)
        self.parent.dock_debug.edit_logger.append(msg)

# ────────────────────────────────────────────────────────────────────────────────



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/dock_debug.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockDebug(object):
    def setupUi(self, DockDebug):
        DockDebug.setObjectName("DockDebug")
        DockDebug.resize(400, 300)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/actions/actions/debug.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        DockDebug.setWindowIcon(icon)
        DockDebug.setAutoFillBackground(False)
        DockDebug.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        DockDebug.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        DockDebug.setAllowedAreas(QtCore.Qt.BottomDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.edit_logger = QtWidgets.QTextBrowser(self.dockWidgetContents)
        self.edit_logger.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.edit_logger.setAutoFormatting(QtWidgets.QTextEdit.AutoAll)
        self.edit_logger.setAcceptRichText(False)
        self.edit_logger.setObjectName("edit_logger")
        self.verticalLayout.addWidget(self.edit_logger)
        self.btn_clear = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btn_clear.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.btn_clear.setAutoDefault(True)
        self.btn_clear.setDefault(True)
        self.btn_clear.setObjectName("btn_clear")
        self.verticalLayout.addWidget(self.btn_clear)
        DockDebug.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockDebug)
        self.btn_clear.released.connect(self.edit_logger.clear)
        QtCore.QMetaObject.connectSlotsByName(DockDebug)

    def retranslateUi(self, DockDebug):
        _translate = QtCore.QCoreApplication.translate
        self.btn_clear.setText(_translate("DockDebug", "Clear"))
import iconos_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/dialog_alarm.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_dialogAlarm(object):
    def setupUi(self, dialogAlarm):
        dialogAlarm.setObjectName("dialogAlarm")
        dialogAlarm.setWindowModality(QtCore.Qt.NonModal)
        dialogAlarm.resize(242, 440)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(dialogAlarm.sizePolicy().hasHeightForWidth())
        dialogAlarm.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/actions/actions/alarms.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        dialogAlarm.setWindowIcon(icon)
        dialogAlarm.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(dialogAlarm)
        self.verticalLayout.setObjectName("verticalLayout")
        self.group_info = QtWidgets.QGroupBox(dialogAlarm)
        self.group_info.setFlat(True)
        self.group_info.setObjectName("group_info")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.group_info)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.ed_exchange = QtWidgets.QLineEdit(self.group_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ed_exchange.sizePolicy().hasHeightForWidth())
        self.ed_exchange.setSizePolicy(sizePolicy)
        self.ed_exchange.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.ed_exchange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_exchange.setReadOnly(True)
        self.ed_exchange.setObjectName("ed_exchange")
        self.verticalLayout_3.addWidget(self.ed_exchange)
        self.ed_market = QtWidgets.QLineEdit(self.group_info)
        self.ed_market.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.ed_market.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_market.setReadOnly(True)
        self.ed_market.setObjectName("ed_market")
        self.verticalLayout_3.addWidget(self.ed_market)
        self.line_2 = QtWidgets.QFrame(self.group_info)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_3.addWidget(self.line_2)
        self.ed_lastPrice = QtWidgets.QLineEdit(self.group_info)
        self.ed_lastPrice.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_lastPrice.setReadOnly(True)
        self.ed_lastPrice.setObjectName("ed_lastPrice")
        self.verticalLayout_3.addWidget(self.ed_lastPrice)
        self.verticalLayout.addWidget(self.group_info)
        self.group_condition = QtWidgets.QGroupBox(dialogAlarm)
        self.group_condition.setFlat(False)
        self.group_condition.setObjectName("group_condition")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.group_condition)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.cb_condition = QtWidgets.QComboBox(self.group_condition)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cb_condition.sizePolicy().hasHeightForWidth())
        self.cb_condition.setSizePolicy(sizePolicy)
        self.cb_condition.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.cb_condition.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.cb_condition.setObjectName("cb_condition")
        self.verticalLayout_2.addWidget(self.cb_condition)
        self.spin_price = QtWidgets.QDoubleSpinBox(self.group_condition)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spin_price.sizePolicy().hasHeightForWidth())
        self.spin_price.setSizePolicy(sizePolicy)
        self.spin_price.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.spin_price.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spin_price.setButtonSymbols(QtWidgets.QAbstractSpinBox.UpDownArrows)
        self.spin_price.setDecimals(8)
        self.spin_price.setObjectName("spin_price")
        self.verticalLayout_2.addWidget(self.spin_price)
        self.ck_autodelete = QtWidgets.QCheckBox(self.group_condition)
        self.ck_autodelete.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.ck_autodelete.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.ck_autodelete.setChecked(True)
        self.ck_autodelete.setObjectName("ck_autodelete")
        self.verticalLayout_2.addWidget(self.ck_autodelete)
        self.verticalLayout.addWidget(self.group_condition)
        self.buttonBox = QtWidgets.QDialogButtonBox(dialogAlarm)
        self.buttonBox.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(dialogAlarm)
        self.buttonBox.accepted.connect(dialogAlarm.accept)
        self.buttonBox.rejected.connect(dialogAlarm.reject)
        QtCore.QMetaObject.connectSlotsByName(dialogAlarm)

    def retranslateUi(self, dialogAlarm):
        _translate = QtCore.QCoreApplication.translate
        dialogAlarm.setWindowTitle(_translate("dialogAlarm", "Alarm"))
        self.group_info.setTitle(_translate("dialogAlarm", "Market"))
        self.ed_lastPrice.setToolTip(_translate("dialogAlarm", "Show last price of this market"))
        self.group_condition.setTitle(_translate("dialogAlarm", "Condition"))
        self.ck_autodelete.setText(_translate("dialogAlarm", "Autodelete when complished"))
import iconos_rc



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qtradingview',
 'qtradingview.alarms',
 'qtradingview.base',
 'qtradingview.debug',
 'qtradingview.markets',
 'qtradingview.models',
 'qtradingview.portfolio',
 'qtradingview.ui']

package_data = \
{'': ['*'],
 'qtradingview': ['i18n/*',
                  'icons/*',
                  'icons/actions/*',
                  'icons/base/*',
                  'icons/exchanges/*',
                  'icons/screenshots/*']}

install_requires = \
['PyQtWebEngine>=5.15.1,<6.0.0',
 'beautifulsoup4>=4.9.3,<5.0.0',
 'cached-property>=1.5.2,<2.0.0',
 'ccxt>=1.37.52,<2.0.0',
 'coloredlogs>=14.0,<15.0',
 'docopt>=0.6.2,<0.7.0',
 'html5lib>=1.1,<2.0',
 'peewee>=3.14.0,<4.0.0',
 'pyqt5-notificator>=1.0.12,<2.0.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['qtradingview = qtradingview.main:run']}

setup_kwargs = {
    'name': 'qtradingview',
    'version': '0.19.4',
    'description': 'PyQt App for TradingView',
    'long_description': '# QTradingView\n\nPyQt App for TradingView.\n\nRecommends simple login to autosave your draws. \n\n\n![Image not found](icons/screenshots/demo.png)  \n\n---\n---\n\n## **Index**\n\n\n- [Features](#Features)\n- [Installation](#Installation)\n- [Usage](#Usage)\n\n---\n---\n***\n\n  \n### **Features**\n\n- Includes the most cryptocurrencies exchanges available in tradingview.\n- Complete lists of available markets, with symbol filter.\n- Favorite and margin lists.\n- Portfolio.\n- Ads remove.\n\n--- \n\n\n### **Installation**\n\nQTradingView needs an environment with Python3 and Qt5\n\n#### ___Prepare environment___\n\n    \n- Install [Anaconda](https://docs.anaconda.com/anaconda/install/)\n\n- Create and active environment.\n    ```\n    conda create -n env_name python=3.7\n    conda activate env_name\n    ```\n\n- Install PyQt5\n    ```\n    conda install -c anaconda pyqt\n    ```\n\n\n#### __QTradingView from source code__\n\n```\n    pip install poetry\n    git clone https://github.com/katmai1/qtradingview\n    cd qtradingview\n\n```\n\n---\n\n### **Usage**\n---\n---\n\n\n#### Install PyQt5 libs using anaconda\n\nCreate and active environment.\n```\nconda create -n env_name python=3.7\nconda activate env_name\n```\n\nInstall PyQt5 and dependencies\n```\nconda install -c anaconda pyqt\n```\n\ninstall\n```\npip install qtradingview\n```\n\n---\n\n## Running from source using Anaconda\n\nCreate and active environment.\n```\nconda create -n env_name python=3.7\nconda activate env_name\n```\n\nInstall PyQt5 and dependencies\n```\nconda install -c anaconda pyqt\npip install -r requirements.txt\n```\n\nRun\n```\npython apprun.py\n```\n\n\n*Can be install without Anaconda if install all PyQt5 dependencies manually.\n\n\n---\n\n\n#### Troubleshot\n\n##### Database issues after an update\n\nProbably the last update does changes into database and this changes are not applied automatically. You can try update tables manually.\n    \n\n```\n- If running from source:\n    python apprun.py --updatedb\n\n- If running compiled release:\n    qtradingview --updatedb\n\n* This function works fine whe running from source code, with a compiled version sometimes not update correctly.\n```\n\nIf issue persist you can delete database to force his create again.\n\n```\n- If running from source:\n    python apprun.py --deletedb\n\n- If running compiled release:\n    qtradingview --deletedb\n```\n\n\n',
    'author': 'katmai',
    'author_email': 'katmai.mobil@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import logging
from PyQt5.QtWidgets import QMessageBox, QLabel, QMainWindow, QTabWidget
from PyQt5.QtCore import Qt, QUrl
from PyQt5.QtMultimedia import QSound

from notificator import notificator
from notificator.alingments import BottomRight

from qtradingview.markets.dock import DockMarkets
from qtradingview.debug.dock import DockDebug, Qlogger
from qtradingview.portfolio.dock import DockPortfolio
from qtradingview.alarms.dock import DockAlarms

from qtradingview.ui.mainwindow_Ui import Ui_MainWindow

from .widgets import CustomWebEnginePage, CustomSplashScreen
from .dialog_config import DialogConfig
from .dialog_about import DialogAbout


# ─── MAIN WINDOW ────────────────────────────────────────────────────────────────

class MainWindow(QMainWindow, Ui_MainWindow):

    def __init__(self, ctx, *args, **kwargs):
        QMainWindow.__init__(self, *args, **kwargs)
        self.setupUi(self)
        #
        self.splash = CustomSplashScreen(self)
        self.ctx = ctx
        self.cfg = self.ctx.settings
        self._notify = notificator()

        # webenginepage
        page = CustomWebEnginePage(self.webview)
        self.webview.setPage(page)

        # signals
        self._docks()
        self._signals()

        # logs
        log_mode = logging.INFO
        if self.ctx.debug:
            log_mode = logging.DEBUG
        qlog = Qlogger(self)
        logging.getLogger().addHandler(qlog)
        logging.getLogger().setLevel(log_mode)

        # carga market inicial
        self._loadInitialConfig()
        self.load_chart(
            self.cfg.value('settings/initial_market'),
            self.cfg.value('settings/initial_exchange')
        )
        self.splash.hide()
        self.static_price = QLabel("- BTC/USDT ")
        self.statusbar.addPermanentWidget(self.static_price)

    def _loadInitialConfig(self):
        # size
        size = self.cfg.value("window/size")
        if size is None:
            self.showMaximized()
        else:
            self.resize(size)
        # position
        position = self.cfg.value("window/pos")
        if position is not None:
            self.move(position)
        # set panels checked
        self.actionDebug.setChecked(self.cfg.value("debug/checked", defaultValue=False, type=bool))
        self.actionMarkets.setChecked(self.cfg.value("markets/checked", defaultValue=True, type=bool))
        self.actionPortfolio.setChecked(self.cfg.value("portfolio/checked", defaultValue=False, type=bool))
        self.actionAlarms.setChecked(self.cfg.value("alarms/checked", defaultValue=False, type=bool))

    def _signals(self):
        """ Define signals """
        self.actionSettings.triggered.connect(self.openDialogSettings)
        self.actionFull_Screen.toggled.connect(self.onActionFullScreen)
        self.actionMarkets.toggled['bool'].connect(self.dock_markets.onActionEvent)
        self.actionDebug.toggled['bool'].connect(self.dock_debug.onActionEvent)
        self.actionPortfolio.toggled['bool'].connect(self.dock_portfolio.onActionEvent)
        self.actionAlarms.toggled['bool'].connect(self.dock_alarms.onActionEvent)
        self.actionAbout.triggered.connect(self.openAboutDialog)

    def _docks(self):
        self.setTabPosition(Qt.BottomDockWidgetArea, QTabWidget.South)
        self.setTabPosition(Qt.LeftDockWidgetArea, QTabWidget.West)
        #
        self.dock_markets = DockMarkets(self)
        self.dock_debug = DockDebug(self)
        self.dock_portfolio = DockPortfolio(self)
        self.dock_alarms = DockAlarms(self)
        #
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dock_markets)
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dock_alarms)
        self.addDockWidget(Qt.BottomDockWidgetArea, self.dock_debug)
        self.addDockWidget(Qt.BottomDockWidgetArea, self.dock_portfolio)
        self.tabifyDockWidget(self.dock_markets, self.dock_alarms)
        self.tabifyDockWidget(self.dock_portfolio, self.dock_debug)

    def _quit(self):
        if self.dock_markets.markets_updater.isRunning():
            self.dock_markets.markets_updater.terminate()
            self.set_text_status(self.tr("Closing background processes..."))
        self.ctx.app.quit()

    def _remember_panels(self):
        self.cfg.setValue("markets/checked", self.actionMarkets.isChecked())
        self.cfg.setValue("markets/list_mode", self.dock_markets.lista_mode)
        self.cfg.setValue("portfolio/checked", self.actionPortfolio.isChecked())
        self.cfg.setValue("debug/checked", self.actionDebug.isChecked())
        self.cfg.setValue("alarms/checked", self.actionAlarms.isChecked())
        self.cfg.setValue("window/size", self.size())
        self.cfg.setValue("window/pos", self.pos())

    # ─── EVENTS ─────────────────────────────────────────────────────────────────────

    # fullscreen
    def onActionFullScreen(self):
        if self.actionFull_Screen.isChecked():
            self.showFullScreen()
        else:
            self.showNormal()
            self.showMaximized()

    def closeEvent(self, event):
        """ Quit app event """
        mbox = QMessageBox(self)
        mbox.setIcon(QMessageBox.Question)
        mbox.setWindowTitle(self.tr('Exit'))
        mbox.setText(self.tr("Do you want quit?"))
        mbox.setStandardButtons(QMessageBox.Yes | QMessageBox.Cancel)
        result = mbox.exec_()
        if int(result) == 16384:
            self._remember_panels()
            self._quit()
        event.ignore()

    # ─── PUBLIC METHODS ──────────────────────────────────────────────

    def notify(self, titulo, texto, tipo="sucess", duracion=None):
        """ Public method to generate notifications """
        QSound.play(":/base/notify")
        if tipo == "sucess":
            self._notify.sucess(titulo, texto, self, BottomRight, duracion=duracion)
        elif tipo == "critical":
            self._notify.critical(titulo, texto, self, BottomRight, duracion=duracion)
        elif tipo == "info":
            self._notify.info(titulo, texto, self, BottomRight, duracion=duracion)
        elif tipo == "warning":
            self._notify.warning(titulo, texto, self, BottomRight, duracion=duracion)
        # else:
        #     self._notify.custom(titulo, texto, self, BottomRight, duracion=duracion)

    def openAboutDialog(self):
        dialog = DialogAbout(self)
        dialog.exec_()

    def set_text_status(self, text, msecs=3000):
        self.statusbar.showMessage(text, msecs=msecs)

    def openDialogSettings(self):
        """ Open dialog settings and reload exchanges if needed """
        dialog = DialogConfig(self)
        if dialog.exec_():
            if dialog.exchanges_is_changed:
                self.dock_markets.markets_updater.first_run = True
                self.dock_markets._load_exchanges()

    # carga un market en la pagina
    def load_chart(self, market, exchange):
        mar, ket = market.split("/")
        url = f"https://es.tradingview.com/chart/?symbol={exchange.upper()}:{mar}{ket}"
        self.webview.setUrl(QUrl(url))
        self.currentExchange = exchange
        self.currentMarket = market



import peewee
import logging
from datetime import datetime
from .base import CustomModel


class Markets(CustomModel):

    exchange = peewee.CharField(null=True)
    symbol = peewee.CharField(null=True)
    tag = peewee.CharField(null=True, default="all")
    favorite = peewee.BooleanField(null=True, default=False)
    #
    limit_min_price = peewee.FloatField(null=True)
    limit_max_price = peewee.FloatField(null=True)
    limit_min_amount = peewee.FloatField(null=True)
    limit_max_amount = peewee.FloatField(null=True)
    limit_min_cost = peewee.FloatField(null=True)
    precision_price = peewee.SmallIntegerField(null=True)
    margin = peewee.BooleanField(null=True, default=False)
    actived = peewee.BooleanField(null=True, default=True)
    # prices
    ask_price = peewee.FloatField(null=True)
    bid_price = peewee.FloatField(null=True)
    last_price = peewee.FloatField(null=True)
    date = peewee.DateTimeField(default=datetime.now)

    class Meta:
        db_table = 'markets'

    def toggle_fav(self):
        self.favorite = not self.favorite
        self.save()

    @classmethod
    def get_all_by_exchange(cls, exchange):
        return cls.select().where(cls.exchange == exchange)

    @classmethod
    def get_symbol_by_exchange(cls, symbol, exchange):
        return cls.get(cls.exchange == exchange.lower(), cls.symbol == symbol)

    @classmethod
    def check_symbol_is_fav(cls, symbol, exchange):
        item = cls.get_symbol_by_exchange(symbol, exchange)
        return item.favorite

    @classmethod
    def check_symbol_is_margin(cls, symbol, exchange):
        item = cls.get_symbol_by_exchange(symbol, exchange)
        return item.margin

    def update_data(self, data):
        self.limit_min_price = data['limits']['price']['min']
        self.limit_max_price = data['limits']['price']['max']
        self.limit_min_amount = data['limits']['amount']['min']
        self.limit_max_amount = data['limits']['amount']['max']
        # pprint(data)
        if data.get('active'):
            self.actived = data['active']
        self.margin = self.get_margin(data)

    def get_margin(self, data):
        try:
            if data.get("margin"):
                return data.get("margin")
            if data.get('info'):
                if data['info'].get("margin"):
                    return data['info'].get("margin")
        except Exception as e:
            logging.debug(e)
            return False

    def update_prices(self, prices):
        self.ask_price = prices['ask']
        self.bid_price = prices['bid']
        self.last_price = prices['last']
        self.date = datetime.now()

    @property
    def since_update(self):
        dif = datetime.now() - self.date
        minutes, seconds = divmod(dif.total_seconds(), 60)
        return [int(minutes), int(seconds)]

# ────────────────────────────────────────────────────────────────────────────────



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/dialog_trade.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogTrade(object):
    def setupUi(self, DialogTrade):
        DialogTrade.setObjectName("DialogTrade")
        DialogTrade.resize(243, 240)
        DialogTrade.setModal(True)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DialogTrade)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(DialogTrade)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_2 = QtWidgets.QLabel(DialogTrade)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.label_3 = QtWidgets.QLabel(DialogTrade)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.label_4 = QtWidgets.QLabel(DialogTrade)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.ed_market = QtWidgets.QLineEdit(DialogTrade)
        self.ed_market.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_market.setReadOnly(True)
        self.ed_market.setObjectName("ed_market")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.ed_market)
        self.combo_type = QtWidgets.QComboBox(DialogTrade)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo_type.sizePolicy().hasHeightForWidth())
        self.combo_type.setSizePolicy(sizePolicy)
        self.combo_type.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.combo_type.setObjectName("combo_type")
        self.combo_type.addItem("")
        self.combo_type.addItem("")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.combo_type)
        self.spin_price = QtWidgets.QDoubleSpinBox(DialogTrade)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spin_price.sizePolicy().hasHeightForWidth())
        self.spin_price.setSizePolicy(sizePolicy)
        self.spin_price.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.spin_price.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spin_price.setButtonSymbols(QtWidgets.QAbstractSpinBox.UpDownArrows)
        self.spin_price.setDecimals(8)
        self.spin_price.setMaximum(9999999.99)
        self.spin_price.setObjectName("spin_price")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.spin_price)
        self.label_5 = QtWidgets.QLabel(DialogTrade)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.spin_amount = QtWidgets.QDoubleSpinBox(DialogTrade)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spin_amount.sizePolicy().hasHeightForWidth())
        self.spin_amount.setSizePolicy(sizePolicy)
        self.spin_amount.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.spin_amount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spin_amount.setDecimals(8)
        self.spin_amount.setMaximum(9999999.99)
        self.spin_amount.setObjectName("spin_amount")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.spin_amount)
        self.ed_exchange = QtWidgets.QLineEdit(DialogTrade)
        self.ed_exchange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ed_exchange.setReadOnly(True)
        self.ed_exchange.setObjectName("ed_exchange")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.ed_exchange)
        self.verticalLayout.addLayout(self.formLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogTrade)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(DialogTrade)
        self.buttonBox.accepted.connect(DialogTrade.accept)
        self.buttonBox.rejected.connect(DialogTrade.close)
        QtCore.QMetaObject.connectSlotsByName(DialogTrade)

    def retranslateUi(self, DialogTrade):
        _translate = QtCore.QCoreApplication.translate
        DialogTrade.setWindowTitle(_translate("DialogTrade", "Trade"))
        self.label.setText(_translate("DialogTrade", "Exchange"))
        self.label_2.setText(_translate("DialogTrade", "Market"))
        self.label_3.setText(_translate("DialogTrade", "Type"))
        self.label_4.setText(_translate("DialogTrade", "Price"))
        self.combo_type.setItemText(0, _translate("DialogTrade", "Long"))
        self.combo_type.setItemText(1, _translate("DialogTrade", "Short"))
        self.label_5.setText(_translate("DialogTrade", "Amount"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/dock_markets.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_dock_markets(object):
    def setupUi(self, dock_markets):
        dock_markets.setObjectName("dock_markets")
        dock_markets.resize(281, 581)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(dock_markets.sizePolicy().hasHeightForWidth())
        dock_markets.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/actions/markets"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        dock_markets.setWindowIcon(icon)
        dock_markets.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        dock_markets.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        dock_markets.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.RightDockWidgetArea)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.combo_exchange = QtWidgets.QComboBox(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.combo_exchange.setFont(font)
        self.combo_exchange.setIconSize(QtCore.QSize(32, 32))
        self.combo_exchange.setObjectName("combo_exchange")
        self.horizontalLayout_2.addWidget(self.combo_exchange)
        self.btn_update = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_update.sizePolicy().hasHeightForWidth())
        self.btn_update.setSizePolicy(sizePolicy)
        self.btn_update.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btn_update.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/actions/update"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_update.setIcon(icon1)
        self.btn_update.setIconSize(QtCore.QSize(32, 32))
        self.btn_update.setObjectName("btn_update")
        self.horizontalLayout_2.addWidget(self.btn_update)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_all = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_all.sizePolicy().hasHeightForWidth())
        self.btn_all.setSizePolicy(sizePolicy)
        self.btn_all.setMinimumSize(QtCore.QSize(0, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.btn_all.setFont(font)
        self.btn_all.setAutoFillBackground(True)
        self.btn_all.setIconSize(QtCore.QSize(24, 24))
        self.btn_all.setCheckable(True)
        self.btn_all.setChecked(False)
        self.btn_all.setAutoExclusive(True)
        self.btn_all.setFlat(False)
        self.btn_all.setObjectName("btn_all")
        self.horizontalLayout.addWidget(self.btn_all)
        self.btn_favorite = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_favorite.sizePolicy().hasHeightForWidth())
        self.btn_favorite.setSizePolicy(sizePolicy)
        self.btn_favorite.setMinimumSize(QtCore.QSize(0, 35))
        self.btn_favorite.setAutoFillBackground(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/base/star"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_favorite.setIcon(icon2)
        self.btn_favorite.setIconSize(QtCore.QSize(24, 24))
        self.btn_favorite.setCheckable(True)
        self.btn_favorite.setChecked(False)
        self.btn_favorite.setAutoExclusive(True)
        self.btn_favorite.setDefault(False)
        self.btn_favorite.setFlat(False)
        self.btn_favorite.setObjectName("btn_favorite")
        self.horizontalLayout.addWidget(self.btn_favorite)
        self.btn_margin = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_margin.sizePolicy().hasHeightForWidth())
        self.btn_margin.setSizePolicy(sizePolicy)
        self.btn_margin.setMinimumSize(QtCore.QSize(0, 35))
        self.btn_margin.setAutoFillBackground(True)
        self.btn_margin.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/base/margin"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btn_margin.setIcon(icon3)
        self.btn_margin.setIconSize(QtCore.QSize(24, 24))
        self.btn_margin.setCheckable(True)
        self.btn_margin.setChecked(False)
        self.btn_margin.setAutoExclusive(True)
        self.btn_margin.setFlat(False)
        self.btn_margin.setObjectName("btn_margin")
        self.horizontalLayout.addWidget(self.btn_margin)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem1)
        self.edit_filtro = QtWidgets.QLineEdit(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.edit_filtro.setFont(font)
        self.edit_filtro.setAcceptDrops(False)
        self.edit_filtro.setInputMethodHints(QtCore.Qt.ImhUppercaseOnly)
        self.edit_filtro.setText("")
        self.edit_filtro.setFrame(True)
        self.edit_filtro.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.edit_filtro.setCursorMoveStyle(QtCore.Qt.LogicalMoveStyle)
        self.edit_filtro.setClearButtonEnabled(True)
        self.edit_filtro.setObjectName("edit_filtro")
        self.verticalLayout.addWidget(self.edit_filtro)
        self.list_markets = QtWidgets.QListWidget(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.list_markets.setFont(font)
        self.list_markets.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.list_markets.setAutoScrollMargin(20)
        self.list_markets.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.list_markets.setTabKeyNavigation(False)
        self.list_markets.setProperty("showDropIndicator", False)
        self.list_markets.setDefaultDropAction(QtCore.Qt.IgnoreAction)
        self.list_markets.setAlternatingRowColors(True)
        self.list_markets.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.list_markets.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.list_markets.setSelectionRectVisible(False)
        self.list_markets.setObjectName("list_markets")
        self.verticalLayout.addWidget(self.list_markets)
        spacerItem2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout.addItem(spacerItem2)
        dock_markets.setWidget(self.dockWidgetContents)

        self.retranslateUi(dock_markets)
        QtCore.QMetaObject.connectSlotsByName(dock_markets)
        dock_markets.setTabOrder(self.edit_filtro, self.list_markets)

    def retranslateUi(self, dock_markets):
        _translate = QtCore.QCoreApplication.translate
        self.combo_exchange.setToolTip(_translate("dock_markets", "Select an exchange"))
        self.btn_update.setToolTip(_translate("dock_markets", "Update markets on db (F5)"))
        self.btn_update.setShortcut(_translate("dock_markets", "F5"))
        self.btn_all.setToolTip(_translate("dock_markets", "Show all markets in list"))
        self.btn_all.setText(_translate("dock_markets", "ALL"))
        self.btn_all.setShortcut(_translate("dock_markets", "Ctrl+A"))
        self.btn_favorite.setToolTip(_translate("dock_markets", "Show only favorite markets"))
        self.btn_favorite.setShortcut(_translate("dock_markets", "Ctrl+F"))
        self.btn_margin.setToolTip(_translate("dock_markets", "Show only margin markets"))
        self.btn_margin.setShortcut(_translate("dock_markets", "Ctrl+M"))
        self.edit_filtro.setToolTip(_translate("dock_markets", "Filter markets by symbol"))
        self.edit_filtro.setPlaceholderText(_translate("dock_markets", "Filter"))
        self.list_markets.setSortingEnabled(True)
import iconos_rc



import ccxt
from PyQt5 import QtCore
from qtradingview.models.markets import Markets


# class UpdateAllPrices(QtCore.QThread):
    
#     onFinished = QtCore.pyqtSignal()
#     infoEvent = QtCore.pyqtSignal(str)

#     def __init__(self, parent=None):
#         super().__init__(parent=parent)
#         self.exchanges_list = parent.mw.config['exchanges']
    
#     def _load_client(self, exchange):
#         self.exchange = exchange.lower()
#         self.client = getattr(ccxt, self.exchange)()
#         self.client.enableRateLimit = True

#     # consigue los markets
#     def _get_markets_by_exchange(self):
#         try:
#             return self.client.fetch_tickers()
#         except Exception:
#             return []

#     # hace la rutina completa con la bd
#     def _one_exchange(self):
#         prices = self._get_markets_by_exchange()
#         for symbol in prices:
#             item, created = Markets.get_or_create(exchange=self.exchange, symbol=symbol)
#             if created:
#                 item.save()
#             item.update_data(markets[symbol])

#     def run(self):
#         for exchange in self.exchanges_list:
#             self._load_client(exchange)
#             self.infoEvent.emit(f"Updating prices of {self.exchange.title()}")
#             self._one_exchange()
#         self.onFinished.emit()

# ────────────────────────────────────────────────────────────────────────────────



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/q/dev/qtradingview/qtradingview/ui/about.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_aboutDialog(object):
    def setupUi(self, aboutDialog):
        aboutDialog.setObjectName("aboutDialog")
        aboutDialog.resize(424, 396)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(aboutDialog.sizePolicy().hasHeightForWidth())
        aboutDialog.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/base/logo"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        aboutDialog.setWindowIcon(icon)
        aboutDialog.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        aboutDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(aboutDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_4 = QtWidgets.QLabel(aboutDialog)
        self.label_4.setMaximumSize(QtCore.QSize(64, 64))
        self.label_4.setText("")
        self.label_4.setPixmap(QtGui.QPixmap(":/base/logo"))
        self.label_4.setScaledContents(True)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout.addWidget(self.label_4)
        self.label_3 = QtWidgets.QLabel(aboutDialog)
        font = QtGui.QFont()
        font.setPointSize(22)
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.label_version = QtWidgets.QLabel(aboutDialog)
        font = QtGui.QFont()
        font.setPointSize(22)
        font.setBold(True)
        font.setWeight(75)
        self.label_version.setFont(font)
        self.label_version.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_version.setObjectName("label_version")
        self.horizontalLayout.addWidget(self.label_version)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.textBrowser = QtWidgets.QTextBrowser(aboutDialog)
        self.textBrowser.setOpenExternalLinks(True)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)
        self.buttonBox = QtWidgets.QDialogButtonBox(aboutDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(aboutDialog)
        self.buttonBox.accepted.connect(aboutDialog.accept)
        self.buttonBox.rejected.connect(aboutDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(aboutDialog)

    def retranslateUi(self, aboutDialog):
        _translate = QtCore.QCoreApplication.translate
        aboutDialog.setWindowTitle(_translate("aboutDialog", "About"))
        self.label_3.setText(_translate("aboutDialog", "QTradingView"))
        self.textBrowser.setHtml(_translate("aboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'Noto Sans\'; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Web: </span><a href=\"https://github.com/katmai1/qtradingview\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt; text-decoration: underline; color:#2980b9;\">https://github.com/katmai1/qtradingview</span></a></p></body></html>"))
import iconos_rc



import logging
import ccxt

from PyQt5 import QtCore
from qtradingview.models.markets import Markets


# ─── MARKETS UPDATER ────────────────────────────────────────────────────────────

class UpdateAllMarkets(QtCore.QThread):

    onFinished = QtCore.pyqtSignal()
    infoEvent = QtCore.pyqtSignal(str)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.first_run = True
        self.exchanges_list = parent.mw.cfg.value('settings/exchanges')

    def _load_client(self, exchange):
        """ Load client for an exchange """
        self.exchange = exchange.lower()
        self.client = getattr(ccxt, self.exchange)()
        self.client.enableRateLimit = True

    def _get_markets_by_exchange(self):
        """ Return all markets of current exchange """
        try:
            return self.client.load_markets()
        except Exception as e:
            logging.error(e)
            return None

    def _get_tickers_by_exchange(self):
        """ Return all prices of current exchange """
        try:
            return self.client.fetch_tickers()
        except Exception as e:
            logging.error(e)
            return None

    def _one_exchange(self):
        """ Run for one exchange """
        if self.first_run:
            self._update_markets_by_exchange()
            self.first_run = False
        self._update_tickers_by_exchange()

    def _update_markets_by_exchange(self):
        """ Update markets of current exchange """
        self.infoEvent.emit(f"Updating markets of {self.exchange.title()}")
        markets = self._get_markets_by_exchange()
        if markets is not None:
            for symbol in markets:
                item, created = Markets.get_or_create(exchange=self.exchange, symbol=symbol)
                item.update_data(markets[symbol])
                item.save()
    
    def _update_tickers_by_exchange(self):
        """ Update prices of current exchange """
        self.infoEvent.emit(f"Updating prices of {self.exchange.title()}")
        prices = self._get_tickers_by_exchange()
        if prices is not None:
            for symbol in prices:
                item, created = Markets.get_or_create(exchange=self.exchange, symbol=symbol)
                item.update_prices(prices[symbol])
                item.save()

    def run(self):
        """ Update all enabled exchange """
        for exchange in self.exchanges_list:
            self._load_client(exchange)
            self._one_exchange()
        self.onFinished.emit()
